//
//  Enemy.m
//  SpaceInvadersTutorial
//
//  Created by Barbara Köhler on 6/14/13.
//  Copyright (c) 2013 Barbara Köhler. All rights reserved.
//

#import "Enemy.h"
#import "Constants.h"
#import "StandardMoveComponent.h"
#import "AnimationComponent.h"

@interface Enemy() {
    id<MoveComponent> moveComponent;
    NSArray* animationComponents;
}

@end

@implementation Enemy

- (id)initWithParent:(SKNode*) parent
{
    self = [super initWithImageNamed:kAlienTestSizeImage];
    if (self) {
        [parent addChild:self];
        moveComponent = [[StandardMoveComponent alloc] initMoveAction:self withDuration: 4.0f];
        self.colorBlendFactor = 1.0f;
        
        NSMutableArray* tempAnimationComponents = [NSMutableArray arrayWithCapacity:5];
        for (int i = 0; i < 5; i++) {
            NSString* baseImageNameFirstPart = [NSString stringWithFormat:kEnemyImageTemplate, i + 1];
            NSString* baseImageName = [baseImageNameFirstPart stringByAppendingString:kAnimationAppendixTemplate];
            SKAction* animation = [[AnimationComponent alloc] initWithTextureAtlas:kAliensAtlas withBaseImageName:baseImageName timePerFrame:0.5f].animation;
            [tempAnimationComponents addObject:animation];
        }
        animationComponents = tempAnimationComponents;
    }
    return self;
}

-(void) resetAtPosition:(int) i {
    self.position = CGPointMake(self.frame.size.width * (i+0.5f), self.parent.frame.size.height + self.frame.size.height/2);
    [self removeAllActions];
}

-(void) move {
    self.hidden = NO;
    [moveComponent move];
    SKColor* randomColor = [[SKColor alloc] initWithRed:MAX(((double)arc4random() / ARC4RANDOM_MAX), 0.3) green:MAX(((double)arc4random() / ARC4RANDOM_MAX), 0.3) blue:MAX(((double)arc4random() / ARC4RANDOM_MAX), 0.3) alpha:255];
    self.color = randomColor;
    
    int randomAnimationNumber = arc4random() % [animationComponents count];
    [self runAction:[animationComponents objectAtIndex:randomAnimationNumber]];
}

-(BOOL) isActive {
    return ([self hasActions] && !self.hidden);
}

-(BOOL) collidesWithPlayerFrame:(CGRect) playerFrame {
    if ([self isActive]) {
        float playerWidth = playerFrame.size.width;
        float enemyWidth = self.frame.size.width;
        float playerCollisionRadius = playerWidth * 0.45f;
        float enemyCollisionRadius = enemyWidth * 0.45f;
        
        CGPoint enemyPosition = self.position;
        CGPoint playerPosition = CGPointMake(playerFrame.origin.x + playerWidth/2,
                                             playerFrame.origin.y + playerFrame.size.height/2);
        
        float maxCollisionDistance = playerCollisionRadius + enemyCollisionRadius;
        float xDistance = enemyPosition.x - playerPosition.x;
        float yDistance = enemyPosition.y - playerPosition.y;
        float actualDistance = sqrt(xDistance * xDistance + yDistance * yDistance);
        
        return (actualDistance < maxCollisionDistance);
    } else {
        return NO;
    }
}

@end
